#!/usr/bin/env python3

# Copyright (C) Francesco Ficarola - All Rights Reserved
# Unauthorized copying of this file, via any medium is strictly prohibited
# Proprietary and confidential
# Written by Francesco Ficarola <francesco.ficarola at uniroma1.it>, December 2020

import sys, argparse, requests, json, urllib3, pyfiglet

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

MGMT_IP_ADDR = 'management_ip_address'
API_KEY = 'api_key'

def printBanner():
    ascii_banner = pyfiglet.figlet_format('   Il Bianconiglio')
    print(f'{bcolors.BOLD}' + ascii_banner + f'{bcolors.ENDC}')

def api_call(command, json_payload, sid):
    url = 'https://' + MGMT_IP_ADDR + '/web_api/' + command
    if sid == '':
        request_headers = {'Content-Type' : 'application/json'}
    else:
        request_headers = {'Content-Type' : 'application/json', 'X-chkp-sid' : sid}
    try:
        r = requests.post(url, data=json.dumps(json_payload), headers=request_headers, verify=False)
        return r.json()
    except Exception as e:
        print(f'\n\n\t> {bcolors.FAIL}Oops! Connection error towards the CP Management Server... Quitting!{bcolors.ENDC}\n\n')
        sys.exit()


def login(key):
    payload = {'api-key': key}
    response = api_call('login', payload, '')
    return response

def main():

    printBanner()

    # https://stackabuse.com/command-line-arguments-in-python/
    # https://docs.python.org/3/howto/argparse.html
    # Initiate the parser
    parser = argparse.ArgumentParser(description='Use this script to search users within Check Point Access Roles.')
    parser.add_argument('user', help='specify user (e.g., sAMAccountName for AD users)')
    parser.add_argument('-i', '--insensitive', help='use case-insenstive comparison', action='store_true')
    args = parser.parse_args()

    search_object = args.user

    login_response = login(API_KEY)
    sid = login_response['sid']

    #print('\n*** Login Success ***\n' + json.dumps(login_response, sort_keys=True, indent=4) + '\n')
    print(f'\n\t> Login: {bcolors.OKGREEN}Success{bcolors.ENDC} (' + sid + ')')

    parameters = {'details-level' : 'full'}
    access_roles = api_call('show-access-roles', parameters, sid)

    #print(json.dumps(access_roles, sort_keys=True, indent=4))
    found_results = {}
    for access_role in access_roles.get('objects'):
        current_access_role = access_role.get('name')
        found_results[current_access_role] = False
        users = access_role.get('users')
        if type(users) == list:
            objects_in_access_role = {}
            for user in users:
                object_name = user.get('name')
                display_name = user.get('display-name') if 'display-name' in user else user.get('name')
                objects_in_access_role[object_name] = display_name
                if args.insensitive:
                    if object_name.lower() == search_object.lower() or object_name.lower() == ('ad_user_' + search_object.lower()) or object_name.lower() == ('ad_group_' + search_object.lower()):
                        found_results[current_access_role] = True
                        print('\n\n\t> Access Role: ' + f'{bcolors.HEADER}' + current_access_role + f'{bcolors.ENDC}\n')
                else:
                    if object_name == search_object or object_name == ('ad_user_' + search_object) or object_name == ('ad_group_' + search_object):
                        found_results[current_access_role] = True
                        print('\n\n\t> Access Role: ' + f'{bcolors.HEADER}' + current_access_role + f'{bcolors.ENDC}\n')
            if found_results[current_access_role] == True:
                print('\t\t>> All objects within this Access Role:\n')
                for key in objects_in_access_role:
                    print(f'\t\t\t{bcolors.OKCYAN}' + key + f'{bcolors.OKBLUE} ('  + objects_in_access_role[key] + ')' + f'{bcolors.ENDC}')

    if all(value == 0 for value in found_results.values()):
        print(f'\n\n\t> {bcolors.FAIL}No user found!{bcolors.ENDC}')


    logout_response = api_call('logout', {}, sid)
    print('\n\n\t> Logout: ' + f'{bcolors.OKGREEN}' + logout_response['message'] + f'{bcolors.ENDC}\n')

if __name__ == '__main__':
    main()
